' Program for reading a correlation/covariance matrix 
' of nuclide yields from the GEF output file and
' to list the values in an extended format.

' How to use this code:
' Cut the section that contains the correlation or the covariance matrix
' from the GEF output file and copy it into a new file.
' Start the program "ReadCorr" (on Linux) or "ReadCorr.exe" (on Windows). 
' It will read the file Corrtest.dat and list the 4-dimensional 
' (Z1, A1, Z2, A2) coordinates and the corresponding matrix values.

Declare Function CC_Count(Chelp As String, Cdiv As String) As Integer
Declare Sub CC_Cut(Chelp As String,CDiv As String,CVhelp() As String, _
               Byref Ndiv as Integer)


Dim As Integer I
Dim As Integer f
Dim As Integer fout
Dim As Byte B_key_found = 0
Dim As Byte B_key = 0
Dim As Byte B_dat = 0
Dim As Byte B_dat_found = 0
Dim As String Chelp,Cdiv
ReDim As String CVhelp(1000)
ReDim As Single RVhelp(1000)
Dim As Integer Ndiv
ReDim As Integer IZlim(2)
IZlim(1) = 1000
IZlim(2) = 0
ReDim As Integer IAlim(100,2)
Dim As Integer IA1,IZ1,IA2,IZ2

Dim As String C_DSN
f = Freefile
Input "Enter file name ",C_DSN
If Instr(C_DSN,".") = 0 Then
  C_DSN = C_DSN + ".dat"
End If
Open C_DSN For Input As #f

fout = Freefile
Open "Out-"+C_DSN for Output As #fout
  
Do
  Line Input #f,Chelp
  Ndiv = CC_Count(Chelp," ")
  If Ndiv = 3 Then
    If B_Key = 0 Then B_key_found = 1
    B_key = 1
    CC_Cut(Chelp," ",CVhelp(),Ndiv)
    For I = 1 To Ndiv
      RVhelp(I) = Val(CVhelp(I))
    Next I      
    If IZlim(1) > RVhelp(1) Then IZlim(1) = RVhelp(1)
    If IZlim(2) < RVhelp(1) Then IZlim(2) = RVhelp(1)
    IAlim(RVhelp(1),1) = RVhelp(2)
    IAlim(RVhelp(1),2) = RVhelp(3)
  Else
    B_key = 0  
  End If 



Loop While B_key_found = 0 Or B_key = 1

Print #fout, "Z limits (of input file, without zero suppression: ";IZlim(1);IZlim(2)
Print #fout, " " 
Print #fout, "A limits (of input file, without zero suppression: "
Print #fout, " Z          A_first         A_last"
For I = IZlim(1) To IZlim(2)
  Print #fout, I,IAlim(I,1),IAlim(I,2)
Next I

Print #fout,
Print #fout, " Output (only matrix elements <> zero):"
Print #fout, "  Z1            A1           Z2            A2           Value"

Dim As Integer I_Elements = 0
IZ1 = IZlim(1)
IA1 = IAlim(IZ1,1) 
IZ2 = IZlim(1)
IA2 = IAlim(IZ2,1) - 1
Do
  Line Input #f,Chelp
  Ndiv = CC_Count(Chelp," ")
  If Ndiv > 50 Then
    B_dat = 1
    B_dat_found = 1
  End If
  If Instr(Chelp,"</") > 0 Then
    B_dat = 0
  End If
  If B_dat = 1 Then
    CC_Cut(Chelp," ",CVhelp(),Ndiv)
    For I = 1 To Ndiv
      RVhelp(I) = Val(CVhelp(I))
      
      IA2 = IA2 + 1
      If IA2 > IAlim(IZ2,2) Then
        IZ2 = IZ2 + 1
        IA2 = IAlim(IZ2,1)
      End If  
      
      IF IZ2 > IZlim(2) Then
        IZ2 = IZlim(1)
        IA2 = IAlim(IZ2,1)
        IA1 = IA1 + 1
      End If
      
      If IA1 > IAlim(IZ1,2) Then
        IZ1 = IZ1 + 1
        IA1 = IAlim(IZ1,1)
      End If  

      I_Elements = I_Elements + 1      
      If RVhelp(I) <> 0 Then _  ' suppress zeroes!
          Print #fout, IZ1,IA1,IZ2,IA2,RVhelp(I)
      
    Next I      
  End If
Loop While ((B_dat = 1) Or (B_dat_found = 0)) And Not(EOF(f))
Print "Conversion completed, ";I_Elements;" matrix elements found."

End

' Functions and Subroutines

Public Function CC_Count(CIn As String, CDiv As String) As Integer
  /' Returns the number of pieces in a string '/
  Dim As Integer I = 0, N
  Dim As String CRest
  CRest = Trim(CIn)  ' remove leading and trailing blanks
  N = 0
  If CRest = "" Then
    Return N
  End If
  I = Instr(CRest,CDiv)
  If I = 0 then
     N = 1
     Return N
  Else
     While I > 0
        N = N + 1
        CRest = Mid(CRest,I+Len(CDiv))
        CRest = Trim(CRest)
        I = Instr(CRest,CDiv)
     Wend 
     N = N + 1
     Return N
  End If  
End Function

Public Sub CC_Cut(CIn As String, CDiv As String, COut() As String, _
               ByRef N As Integer) 
  /' Cuts a string into pieces with CDiv as divider '/ 
  /' Special treatment for blank as devider: 
       multiple blanks count as one divider '/              
  Dim As Integer I = 0
  Dim As String CRest
  CRest = Trim(CIn)  ' remove leading and trailing blanks
  N = 0
  If CRest = "" Then
    Return
  End If
  I = Instr(CRest,CDiv)
  If I = 0 then
     N = 1
     COut(1) = CIn
     Return
  Else
     While I > 0
        N = N + 1
        If N > Ubound(COut) Then 
          N = N - 1
          Print "<E> Dimension of COut too small in CC_cut(";CIn;")" 
        End if
        COut(N) = Mid(CRest,1,I-1)
        CRest = Mid(CRest,I+Len(CDiv))
        CRest = Trim(CRest)
        I = Instr(CRest,CDiv)
     Wend 
     N = N + 1
     If N > Ubound(COut) Then
          N = N - 1
          Print "<E> Dimension of COut too small in CC_cut(";CIn;")" 
     End if
     COut(N) = CRest
     Return
  End if
End Sub
